/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib;

import java.util.Collection;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Enthält Methoden zum Zugriff auf die Hierarchie der Systemobjekte im
 * Datenkatalog.
 *
 * @author BitCtrl Systems GmbH, Schumann, Peuker
 */
public interface Hierarchie {

	/**
	 * liefert das Systemobjekt, an dem die Parameter für die übergebene
	 * Spezifikation definiert sind.<br>
	 * Wenn das Objekt nicht ermittelt werden konnte, wird der Wert {@code null}
	 * geliefert.
	 *
	 * @param info
	 *            die Definition des gesuchten Parameters
	 * @return das Objekt
	 */
	SystemObject getParameterWurzel(ParameterInfo info);

	/**
	 * liefert die Systemobjekt, an dem die Parameter für das übergebene Feld
	 * von Spezifikationen definiert sind.<br>
	 * Das Feld wird in der gleichen Reiehenfolge gefüllt, in der auch die
	 * Parameterdefinitionen übergeben wurden.<br>
	 * Falls das Wurzelobjekt nicht ermittelt werden konnten, wird der Wert
	 * {@code null} geliefert.
	 *
	 * @param info
	 *            die Definitionen der gesuchten Parameters
	 * @return die Feld mit den Objekt
	 */
	SystemObject[] getParameterWurzel(ParameterInfo[] info);

	/**
	 * liefert die Systemobjekt, an dem die Parameter für die übergebene
	 * Collection von Spezifikationen definiert sind.<br>
	 * Das Ergebnis ist eine Schlüsselmenge, auf die mit der
	 * Parameterspezifikation zugegriffen werden kann.<br>
	 * Falls die Wurzelobjekte nicht ermittelt werden konnten, wird der Wert
	 * {@code null} geliefert.
	 *
	 * @param info
	 *            die Definitionen der gesuchten Parameters
	 * @return die Feld mit den Objekt
	 */
	Map<ParameterInfo, SystemObject> getParameterWurzel(Collection<ParameterInfo> info);

	/**
	 * liefert eine Menge von Objekten, deren Parameter potentiell an dem Objekt
	 * entsprechend der übergebenen Parameterspezifikation angegeben werden
	 * können.<br>
	 * Es wird nicht überprüft, ob die Objekte durch lokale Parameter näher
	 * definiert sind.<br>
	 * Wenn die Objekte nicht ermittelt werden konnten, wird der Wert
	 * {@code null} geliefert.
	 *
	 *
	 * @param info
	 *            die Parameterspezifikation
	 * @return die Menge der mit dieser Parameterspezifikation beeinflussbaren
	 *         Objekte
	 */
	Collection<SystemObject> getAenderbareObjekte(ParameterInfo info);
}
