/*
 * Copyright 2017-2018 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.kappich.pat.gnd.
 *
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.rnPlugin;

import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import de.kappich.pat.gnd.properties.PropertyValue;

import java.util.prefs.Preferences;

/**
 * {@code RnFormat} ist eine Enumeration aller möglichen Formate der Routennummernformaten.
 * Die einzelnen Werte besiztzen eine eindeutige Ordinalzahl.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
public enum RnFormat implements PropertyValue {
	RN_SMALL("Kleine Autobahnschilder", 0),
	RN_BIG("Große Autobahnschilder", 1);

	private final String _description;
	private final int _ordinal;

	RnFormat(final String description, final int ordinal) {
		_description = description;
		_ordinal = ordinal;
	}

	@Override
	public void putPreferences(final Preferences prefs) {
		prefs.putInt("PROPERTY_VALUE", getOrdinal());
	}

	/**
	 * Gibt das {@code StatFormat}-Objekt zu der übergebenen Ordinalzahl zurück, bzw. {@code null},
	 * wenn kein solches Objekt existiert.
	 *
	 * @param ordinal die Ordinalzahl
	 * @return das Objekt
	 */
	@Nullable
	public static RnFormat getFormat(int ordinal) {
		for(RnFormat format : RnFormat.values()) {
			if(format.getOrdinal() == ordinal)
				return format;
		}
		return null;
	}

	/**
	 * Gibt die Ordinalzahl des {@code StatFormat}-Objekts zurück.
	 *
	 * @return die Ordinalzahl
	 */
	public int getOrdinal() {
		return _ordinal;
	}

	@Override
	public String toString() {
		return _description;
	}
}
